@@ -27,7 +27,7 @@ PARROT_CANNOT_RETURN_NULL
 PMC * Parrot_ns_find_global_from_op(PARROT_INTERP,
     ARGIN(PMC *ns),
     ARGIN_NULLOK(STRING *globalname),
-    ARGIN_NULLOK(void *next))
+    void *next)
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
@@ -83,6 +83,11 @@ PMC * Parrot_ns_get_namespace_keyed_str(PARROT_INTERP,
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
+PARROT_EXPORT
+PARROT_PURE_FUNCTION
+Parrot_PMC Parrot_ns_get_root_namespace(PARROT_INTERP)
+        __attribute__nonnull__(1);
+
 PARROT_EXPORT
 PARROT_WARN_UNUSED_RESULT
 PARROT_CANNOT_RETURN_NULL
@@ -156,6 +161,8 @@ void Parrot_ns_store_sub(PARROT_INTERP, ARGIN(PMC *sub_pmc))
      __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp) \
     , PARROT_ASSERT_ARG(base_ns))
+#define ASSERT_ARGS_Parrot_ns_get_root_namespace __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
+       PARROT_ASSERT_ARG(interp))
 #define ASSERT_ARGS_Parrot_ns_make_namespace_autobase \
      __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp) \
