@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2012, Parrot Foundation.
+ * Copyright (C) 2001-2015, Parrot Foundation.
  */
 
 /* packfile.h
@@ -32,7 +32,8 @@
 /* NaN difference, but patches welcome */
 #define FLOATTYPE_16MIPS      3
 #define FLOATTYPE_16MIPS_NAME "MIPS 16 byte long double"
-/* See http://www.ncsa.uiuc.edu/UserInfo/Resources/Hardware/IBMp690/IBM/usr/share/man/info/en_US/a_doc_lib/aixprggd/genprogc/128bit_long_double_floating-point_datatype.htm */
+/* See http://www.ncsa.uiuc.edu/UserInfo/Resources/Hardware/IBMp690/IBM/usr/share/ ...
+ * man/info/en_US/a_doc_lib/aixprggd/genprogc/128bit_long_double_floating-point_datatype.htm */
 #define FLOATTYPE_16AIX       4
 #define FLOATTYPE_16AIX_NAME  "AIX 16 byte long double"
 /* IEEE-754 old and tiny, yet unsupported */
@@ -166,7 +167,8 @@ typedef struct PackFile_Segment {
     opcode_t           *data;           /* oparray e.g. bytecode */
 } PackFile_Segment;
 
-typedef INTVAL (*PackFile_map_segments_func_t)(PARROT_INTERP, ARGMOD(PackFile_Segment *seg), ARGIN_NULLOK(void *user_data));
+typedef INTVAL (*PackFile_map_segments_func_t)(PARROT_INTERP,
+        ARGMOD(PackFile_Segment *seg), ARGIN_NULLOK(void *user_data));
 
 typedef struct PackFile_ConstTagPair {
     opcode_t tag_idx;
@@ -236,7 +238,7 @@ typedef struct PackFile_Debug {
 #define ANN_ENTRY_OFF 0
 #define ANN_ENTRY_VAL 1
 
-/* &gen_from_def(packfile_annotation_key_type.pasm) */
+/* &gen_from_def(packfile_annotation_keytype.pasm) */
 
 /* Key types for annotation segment. */
 typedef enum {
@@ -343,8 +345,6 @@ FLOATVAL PF_fetch_number(
     ARGIN(const opcode_t **stream))
         __attribute__nonnull__(2);
 
-PARROT_WARN_UNUSED_RESULT
-PARROT_CANNOT_RETURN_NULL
 opcode_t PF_fetch_opcode(
     ARGIN_NULLOK(const PackFile *pf),
     ARGMOD(const opcode_t **stream))
@@ -478,13 +478,11 @@ opcode_t* PF_store_string(ARGOUT(opcode_t *cursor), ARGIN(const STRING *s))
 /* HEADERIZER BEGIN: frontend/pbc_dump/packdump.c */
 /* Don't modify between HEADERIZER BEGIN / HEADERIZER END.  Your changes will be lost. */
 
-PARROT_EXPORT
-void PackFile_ConstTable_dump(PARROT_INTERP,
-    ARGIN(const PackFile_ConstTable *self))
+void pf_const_dump(PARROT_INTERP, ARGIN(const PackFile_ConstTable *self))
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
-#define ASSERT_ARGS_PackFile_ConstTable_dump __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
+#define ASSERT_ARGS_pf_const_dump __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp) \
     , PARROT_ASSERT_ARG(self))
 /* Don't modify between HEADERIZER BEGIN / HEADERIZER END.  Your changes will be lost. */
@@ -494,24 +492,16 @@ void PackFile_ConstTable_dump(PARROT_INTERP,
 /* Don't modify between HEADERIZER BEGIN / HEADERIZER END.  Your changes will be lost. */
 
 PARROT_EXPORT
+PARROT_DEPRECATED
 void do_sub_pragmas(PARROT_INTERP,
     ARGIN(PMC *pfpmc),
     pbc_action_enum_t action,
-    ARGIN_NULLOK(PMC *eval_pmc))
+    PMC *eval_pmc)
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
 PARROT_EXPORT
-void PackFile_add_segment(PARROT_INTERP,
-    ARGMOD(PackFile_Directory *dir),
-    ARGMOD(PackFile_Segment *seg))
-        __attribute__nonnull__(1)
-        __attribute__nonnull__(2)
-        __attribute__nonnull__(3)
-        FUNC_MODIFIES(*dir)
-        FUNC_MODIFIES(*seg);
-
-PARROT_EXPORT
+PARROT_DEPRECATED
 void PackFile_Annotations_add_entry(PARROT_INTERP,
     ARGMOD(PackFile_Annotations *self),
     opcode_t offset,
@@ -523,43 +513,27 @@ void PackFile_Annotations_add_entry(PARROT_INTERP,
         FUNC_MODIFIES(*self);
 
 PARROT_EXPORT
+PARROT_DEPRECATED
 void PackFile_destroy(PARROT_INTERP, ARGMOD(PackFile *pf))
         __attribute__nonnull__(1)
         __attribute__nonnull__(2)
         FUNC_MODIFIES(*pf);
 
 PARROT_EXPORT
-PARROT_WARN_UNUSED_RESULT
-PARROT_CAN_RETURN_NULL
-PackFile_Segment * PackFile_find_segment(PARROT_INTERP,
-    ARGIN_NULLOK(PackFile_Directory *dir),
-    ARGIN(const STRING *name),
-    int sub_dir)
-        __attribute__nonnull__(1)
-        __attribute__nonnull__(3);
-
-PARROT_EXPORT
-void PackFile_fixup_subs(PARROT_INTERP,
-    pbc_action_enum_t what,
-    ARGIN_NULLOK(PMC *eval))
+PARROT_DEPRECATED
+void PackFile_fixup_subs(PARROT_INTERP, pbc_action_enum_t what, PMC *eval)
         __attribute__nonnull__(1);
 
-PARROT_EXPORT
-INTVAL PackFile_map_segments(PARROT_INTERP,
-    ARGIN(const PackFile_Directory *dir),
-    PackFile_map_segments_func_t callback,
-    ARGIN_NULLOK(void *user_data))
-        __attribute__nonnull__(1)
-        __attribute__nonnull__(2);
-
 PARROT_EXPORT
 PARROT_WARN_UNUSED_RESULT
 PARROT_CANNOT_RETURN_NULL
+PARROT_DEPRECATED
 PackFile * PackFile_new(PARROT_INTERP, INTVAL is_mapped)
         __attribute__nonnull__(1);
 
 PARROT_EXPORT
 PARROT_WARN_UNUSED_RESULT
+PARROT_DEPRECATED
 opcode_t PackFile_unpack(PARROT_INTERP,
     ARGMOD(PackFile *self),
     ARGIN(const opcode_t *packed),
@@ -570,6 +544,7 @@ opcode_t PackFile_unpack(PARROT_INTERP,
         FUNC_MODIFIES(*self);
 
 PARROT_EXPORT
+PARROT_DEPRECATED
 void Parrot_debug_add_mapping(PARROT_INTERP,
     ARGMOD(PackFile_Debug *debug),
     opcode_t offset,
@@ -582,6 +557,7 @@ void Parrot_debug_add_mapping(PARROT_INTERP,
 PARROT_EXPORT
 PARROT_WARN_UNUSED_RESULT
 PARROT_CANNOT_RETURN_NULL
+PARROT_DEPRECATED
 STRING * Parrot_debug_pc_to_filename(PARROT_INTERP,
     ARGIN(const PackFile_Debug *debug),
     opcode_t pc)
@@ -600,6 +576,7 @@ void Parrot_load_language(PARROT_INTERP, ARGIN_NULLOK(STRING *lang_name))
 PARROT_EXPORT
 PARROT_WARN_UNUSED_RESULT
 PARROT_CANNOT_RETURN_NULL
+PARROT_DEPRECATED
 PackFile_Debug * Parrot_new_debug_seg(PARROT_INTERP,
     ARGMOD(PackFile_ByteCode *cs),
     size_t size)
@@ -607,6 +584,17 @@ PackFile_Debug * Parrot_new_debug_seg(PARROT_INTERP,
         __attribute__nonnull__(2)
         FUNC_MODIFIES(*cs);
 
+PARROT_EXPORT
+void Parrot_pf_annotations_add_entry(PARROT_INTERP,
+    ARGMOD(PackFile_Annotations *self),
+    opcode_t offset,
+    opcode_t key,
+    opcode_t type,
+    opcode_t value)
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
+        FUNC_MODIFIES(*self);
+
 PARROT_EXPORT
 PARROT_WARN_UNUSED_RESULT
 PARROT_CANNOT_RETURN_NULL
@@ -618,6 +606,25 @@ PackFile_ByteCode * Parrot_pf_create_default_segments(PARROT_INTERP,
         __attribute__nonnull__(2)
         __attribute__nonnull__(3);
 
+PARROT_EXPORT
+void Parrot_pf_debug_add_mapping(PARROT_INTERP,
+    ARGMOD(PackFile_Debug *debug),
+    opcode_t offset,
+    ARGIN(STRING *filename))
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
+        __attribute__nonnull__(4)
+        FUNC_MODIFIES(*debug);
+
+PARROT_EXPORT
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
+STRING * Parrot_pf_debug_pc_to_filename(PARROT_INTERP,
+    ARGIN(const PackFile_Debug *debug),
+    opcode_t pc)
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2);
+
 PARROT_EXPORT
 PARROT_CANNOT_RETURN_NULL
 PackFile * Parrot_pf_deserialize(PARROT_INTERP, ARGIN(STRING *str))
@@ -640,6 +647,10 @@ void Parrot_pf_execute_bytecode_program(PARROT_INTERP,
         FUNC_MODIFIES(*pbc)
         FUNC_MODIFIES(*args);
 
+PARROT_EXPORT
+void Parrot_pf_fixup_subs(PARROT_INTERP, pbc_action_enum_t what, PMC *eval)
+        __attribute__nonnull__(1);
+
 PARROT_EXPORT
 PARROT_CANNOT_RETURN_NULL
 PackFile_Annotations * Parrot_pf_get_annotations_segment(PARROT_INTERP,
@@ -677,6 +688,16 @@ PARROT_CANNOT_RETURN_NULL
 PackFile * Parrot_pf_new(PARROT_INTERP, INTVAL is_mapped)
         __attribute__nonnull__(1);
 
+PARROT_EXPORT
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
+PackFile_Debug * Parrot_pf_new_debug_segment(PARROT_INTERP,
+    ARGMOD(PackFile_ByteCode *cs),
+    size_t size)
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
+        FUNC_MODIFIES(*cs);
+
 PARROT_EXPORT
 void Parrot_pf_prepare_packfile_init(PARROT_INTERP,
     ARGIN(PMC * const pfpmc))
@@ -712,6 +733,15 @@ void Parrot_pf_set_current_packfile(PARROT_INTERP, ARGIN(PMC *pbc))
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
+PARROT_EXPORT
+PARROT_CANNOT_RETURN_NULL
+PMC * Parrot_pf_single_sub_by_tag(PARROT_INTERP,
+    ARGIN(PMC * pfpmc),
+    ARGIN(STRING * flag))
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
+        __attribute__nonnull__(3);
+
 PARROT_EXPORT
 PARROT_CANNOT_RETURN_NULL
 PMC * Parrot_pf_subs_by_tag(PARROT_INTERP,
@@ -721,6 +751,15 @@ PMC * Parrot_pf_subs_by_tag(PARROT_INTERP,
         __attribute__nonnull__(2)
         __attribute__nonnull__(3);
 
+PARROT_EXPORT
+PARROT_IGNORABLE_RESULT
+PARROT_CANNOT_RETURN_NULL
+PackFile_ByteCode * Parrot_pf_switch_to_cs(PARROT_INTERP,
+    ARGIN(PackFile_ByteCode *new_cs),
+    int really)
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2);
+
 PARROT_EXPORT
 void Parrot_pf_tag_constant(PARROT_INTERP,
     ARGIN(PackFile_ConstTable *ct),
@@ -729,6 +768,17 @@ void Parrot_pf_tag_constant(PARROT_INTERP,
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
+PARROT_EXPORT
+PARROT_WARN_UNUSED_RESULT
+opcode_t Parrot_pf_unpack(PARROT_INTERP,
+    ARGMOD(PackFile *self),
+    ARGIN(const opcode_t *packed),
+    size_t packed_size)
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
+        __attribute__nonnull__(3)
+        FUNC_MODIFIES(*self);
+
 PARROT_EXPORT
 void Parrot_pf_write_pbc_file(PARROT_INTERP,
     ARGIN(PMC *pf_pmc),
@@ -740,20 +790,13 @@ void Parrot_pf_write_pbc_file(PARROT_INTERP,
 PARROT_EXPORT
 PARROT_IGNORABLE_RESULT
 PARROT_CANNOT_RETURN_NULL
+PARROT_DEPRECATED
 PackFile_ByteCode * Parrot_switch_to_cs(PARROT_INTERP,
     ARGIN(PackFile_ByteCode *new_cs),
     int really)
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
-PARROT_CANNOT_RETURN_NULL
-PMC * PackFile_Annotations_lookup(PARROT_INTERP,
-    ARGIN(PackFile_Annotations *self),
-    opcode_t offset,
-    ARGIN_NULLOK(STRING *name))
-        __attribute__nonnull__(1)
-        __attribute__nonnull__(2);
-
 PARROT_CANNOT_RETURN_NULL
 PARROT_WARN_UNUSED_RESULT
 PMC * Parrot_pf_all_subs(PARROT_INTERP, ARGIN(PMC *pfpmc))
@@ -772,8 +815,16 @@ PMC * Parrot_pf_all_tags_list(PARROT_INTERP, ARGIN(PMC * pfpmc))
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
-PARROT_PURE_FUNCTION
 PARROT_CANNOT_RETURN_NULL
+PMC * Parrot_pf_annotations_lookup(PARROT_INTERP,
+    ARGIN(PackFile_Annotations *self),
+    opcode_t offset,
+    ARGIN_NULLOK(STRING *name))
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2);
+
+PARROT_PURE_FUNCTION
+PARROT_CAN_RETURN_NULL
 PackFile_ByteCode * Parrot_pf_get_current_code_segment(PARROT_INTERP)
         __attribute__nonnull__(1);
 
@@ -794,10 +845,6 @@ void Parrot_pf_mark_packfile(PARROT_INTERP, ARGMOD_NULLOK(PackFile * pf))
 #define ASSERT_ARGS_do_sub_pragmas __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp) \
     , PARROT_ASSERT_ARG(pfpmc))
-#define ASSERT_ARGS_PackFile_add_segment __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
-       PARROT_ASSERT_ARG(interp) \
-    , PARROT_ASSERT_ARG(dir) \
-    , PARROT_ASSERT_ARG(seg))
 #define ASSERT_ARGS_PackFile_Annotations_add_entry \
      __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp) \
@@ -805,14 +852,8 @@ void Parrot_pf_mark_packfile(PARROT_INTERP, ARGMOD_NULLOK(PackFile * pf))
 #define ASSERT_ARGS_PackFile_destroy __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp) \
     , PARROT_ASSERT_ARG(pf))
-#define ASSERT_ARGS_PackFile_find_segment __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
-       PARROT_ASSERT_ARG(interp) \
-    , PARROT_ASSERT_ARG(name))
 #define ASSERT_ARGS_PackFile_fixup_subs __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp))
-#define ASSERT_ARGS_PackFile_map_segments __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
-       PARROT_ASSERT_ARG(interp) \
-    , PARROT_ASSERT_ARG(dir))
 #define ASSERT_ARGS_PackFile_new __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp))
 #define ASSERT_ARGS_PackFile_unpack __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
@@ -833,11 +874,23 @@ void Parrot_pf_mark_packfile(PARROT_INTERP, ARGMOD_NULLOK(PackFile * pf))
 #define ASSERT_ARGS_Parrot_new_debug_seg __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp) \
     , PARROT_ASSERT_ARG(cs))
+#define ASSERT_ARGS_Parrot_pf_annotations_add_entry \
+     __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
+       PARROT_ASSERT_ARG(interp) \
+    , PARROT_ASSERT_ARG(self))
 #define ASSERT_ARGS_Parrot_pf_create_default_segments \
      __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp) \
     , PARROT_ASSERT_ARG(pf_pmc) \
     , PARROT_ASSERT_ARG(file_name))
+#define ASSERT_ARGS_Parrot_pf_debug_add_mapping __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
+       PARROT_ASSERT_ARG(interp) \
+    , PARROT_ASSERT_ARG(debug) \
+    , PARROT_ASSERT_ARG(filename))
+#define ASSERT_ARGS_Parrot_pf_debug_pc_to_filename \
+     __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
+       PARROT_ASSERT_ARG(interp) \
+    , PARROT_ASSERT_ARG(debug))
 #define ASSERT_ARGS_Parrot_pf_deserialize __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp) \
     , PARROT_ASSERT_ARG(str))
@@ -849,6 +902,8 @@ void Parrot_pf_mark_packfile(PARROT_INTERP, ARGMOD_NULLOK(PackFile * pf))
        PARROT_ASSERT_ARG(interp) \
     , PARROT_ASSERT_ARG(pbc) \
     , PARROT_ASSERT_ARG(args))
+#define ASSERT_ARGS_Parrot_pf_fixup_subs __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
+       PARROT_ASSERT_ARG(interp))
 #define ASSERT_ARGS_Parrot_pf_get_annotations_segment \
      __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp) \
@@ -866,6 +921,9 @@ void Parrot_pf_mark_packfile(PARROT_INTERP, ARGMOD_NULLOK(PackFile * pf))
     , PARROT_ASSERT_ARG(file))
 #define ASSERT_ARGS_Parrot_pf_new __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp))
+#define ASSERT_ARGS_Parrot_pf_new_debug_segment __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
+       PARROT_ASSERT_ARG(interp) \
+    , PARROT_ASSERT_ARG(cs))
 #define ASSERT_ARGS_Parrot_pf_prepare_packfile_init \
      __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp) \
@@ -886,13 +944,24 @@ void Parrot_pf_mark_packfile(PARROT_INTERP, ARGMOD_NULLOK(PackFile * pf))
      __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp) \
     , PARROT_ASSERT_ARG(pbc))
+#define ASSERT_ARGS_Parrot_pf_single_sub_by_tag __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
+       PARROT_ASSERT_ARG(interp) \
+    , PARROT_ASSERT_ARG(pfpmc) \
+    , PARROT_ASSERT_ARG(flag))
 #define ASSERT_ARGS_Parrot_pf_subs_by_tag __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp) \
     , PARROT_ASSERT_ARG(pfpmc) \
     , PARROT_ASSERT_ARG(flag))
+#define ASSERT_ARGS_Parrot_pf_switch_to_cs __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
+       PARROT_ASSERT_ARG(interp) \
+    , PARROT_ASSERT_ARG(new_cs))
 #define ASSERT_ARGS_Parrot_pf_tag_constant __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp) \
     , PARROT_ASSERT_ARG(ct))
+#define ASSERT_ARGS_Parrot_pf_unpack __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
+       PARROT_ASSERT_ARG(interp) \
+    , PARROT_ASSERT_ARG(self) \
+    , PARROT_ASSERT_ARG(packed))
 #define ASSERT_ARGS_Parrot_pf_write_pbc_file __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp) \
     , PARROT_ASSERT_ARG(pf_pmc) \
@@ -900,9 +969,6 @@ void Parrot_pf_mark_packfile(PARROT_INTERP, ARGMOD_NULLOK(PackFile * pf))
 #define ASSERT_ARGS_Parrot_switch_to_cs __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp) \
     , PARROT_ASSERT_ARG(new_cs))
-#define ASSERT_ARGS_PackFile_Annotations_lookup __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
-       PARROT_ASSERT_ARG(interp) \
-    , PARROT_ASSERT_ARG(self))
 #define ASSERT_ARGS_Parrot_pf_all_subs __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp) \
     , PARROT_ASSERT_ARG(pfpmc))
@@ -912,6 +978,9 @@ void Parrot_pf_mark_packfile(PARROT_INTERP, ARGMOD_NULLOK(PackFile * pf))
 #define ASSERT_ARGS_Parrot_pf_all_tags_list __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp) \
     , PARROT_ASSERT_ARG(pfpmc))
+#define ASSERT_ARGS_Parrot_pf_annotations_lookup __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
+       PARROT_ASSERT_ARG(interp) \
+    , PARROT_ASSERT_ARG(self))
 #define ASSERT_ARGS_Parrot_pf_get_current_code_segment \
      __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp))
@@ -931,6 +1000,7 @@ void Parrot_pf_mark_packfile(PARROT_INTERP, ARGMOD_NULLOK(PackFile * pf))
 /* Don't modify between HEADERIZER BEGIN / HEADERIZER END.  Your changes will be lost. */
 
 PARROT_EXPORT
+PARROT_DEPRECATED
 PARROT_WARN_UNUSED_RESULT
 PARROT_CANNOT_RETURN_NULL
 opcode_t * PackFile_ConstTable_pack(PARROT_INTERP,
@@ -943,6 +1013,7 @@ opcode_t * PackFile_ConstTable_pack(PARROT_INTERP,
         FUNC_MODIFIES(*cursor);
 
 PARROT_EXPORT
+PARROT_DEPRECATED
 size_t PackFile_ConstTable_pack_size(PARROT_INTERP,
     ARGMOD(PackFile_Segment *seg))
         __attribute__nonnull__(1)
@@ -950,12 +1021,14 @@ size_t PackFile_ConstTable_pack_size(PARROT_INTERP,
         FUNC_MODIFIES(*seg);
 
 PARROT_EXPORT
+PARROT_DEPRECATED
 int PackFile_ConstTable_rlookup_num(PARROT_INTERP,
     ARGIN(const PackFile_ConstTable *ct),
     FLOATVAL n)
         __attribute__nonnull__(2);
 
 PARROT_EXPORT
+PARROT_DEPRECATED
 int PackFile_ConstTable_rlookup_pmc(PARROT_INTERP,
     ARGIN(PackFile_ConstTable *ct),
     ARGIN(PMC *v),
@@ -970,6 +1043,7 @@ int PackFile_ConstTable_rlookup_pmc(PARROT_INTERP,
         FUNC_MODIFIES(*idx);
 
 PARROT_EXPORT
+PARROT_DEPRECATED
 int PackFile_ConstTable_rlookup_str(PARROT_INTERP,
     ARGIN(const PackFile_ConstTable *ct),
     ARGIN(STRING *s))
@@ -978,6 +1052,7 @@ int PackFile_ConstTable_rlookup_str(PARROT_INTERP,
         __attribute__nonnull__(3);
 
 PARROT_EXPORT
+PARROT_DEPRECATED
 void PackFile_pack(PARROT_INTERP,
     ARGMOD(PackFile *self),
     ARGOUT(opcode_t *cursor))
@@ -988,11 +1063,75 @@ void PackFile_pack(PARROT_INTERP,
         FUNC_MODIFIES(*cursor);
 
 PARROT_EXPORT
+PARROT_DEPRECATED
 size_t PackFile_pack_size(PARROT_INTERP, ARGMOD(PackFile *self))
         __attribute__nonnull__(1)
         __attribute__nonnull__(2)
         FUNC_MODIFIES(*self);
 
+PARROT_EXPORT
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
+opcode_t * Parrot_pf_ConstTable_pack(PARROT_INTERP,
+    ARGMOD(PackFile_Segment *seg),
+    ARGOUT(opcode_t *cursor))
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
+        __attribute__nonnull__(3)
+        FUNC_MODIFIES(*seg)
+        FUNC_MODIFIES(*cursor);
+
+PARROT_EXPORT
+size_t Parrot_pf_ConstTable_pack_size(PARROT_INTERP,
+    ARGMOD(PackFile_Segment *seg))
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
+        FUNC_MODIFIES(*seg);
+
+PARROT_EXPORT
+int Parrot_pf_ConstTable_rlookup_num(PARROT_INTERP,
+    ARGIN(const PackFile_ConstTable *ct),
+    FLOATVAL n)
+        __attribute__nonnull__(2);
+
+PARROT_EXPORT
+int Parrot_pf_ConstTable_rlookup_pmc(PARROT_INTERP,
+    ARGIN(PackFile_ConstTable *ct),
+    ARGIN(PMC *v),
+    ARGOUT(INTVAL *constno),
+    ARGOUT(INTVAL *idx))
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
+        __attribute__nonnull__(3)
+        __attribute__nonnull__(4)
+        __attribute__nonnull__(5)
+        FUNC_MODIFIES(*constno)
+        FUNC_MODIFIES(*idx);
+
+PARROT_EXPORT
+int Parrot_pf_ConstTable_rlookup_str(PARROT_INTERP,
+    ARGIN(const PackFile_ConstTable *ct),
+    ARGIN(STRING *s))
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
+        __attribute__nonnull__(3);
+
+PARROT_EXPORT
+void Parrot_pf_pack(PARROT_INTERP,
+    ARGMOD(PackFile *self),
+    ARGOUT(opcode_t *cursor))
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
+        __attribute__nonnull__(3)
+        FUNC_MODIFIES(*self)
+        FUNC_MODIFIES(*cursor);
+
+PARROT_EXPORT
+size_t Parrot_pf_pack_size(PARROT_INTERP, ARGMOD(PackFile *self))
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
+        FUNC_MODIFIES(*self);
+
 #define ASSERT_ARGS_PackFile_ConstTable_pack __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp) \
     , PARROT_ASSERT_ARG(seg) \
@@ -1022,6 +1161,36 @@ size_t PackFile_pack_size(PARROT_INTERP, ARGMOD(PackFile *self))
 #define ASSERT_ARGS_PackFile_pack_size __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp) \
     , PARROT_ASSERT_ARG(self))
+#define ASSERT_ARGS_Parrot_pf_ConstTable_pack __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
+       PARROT_ASSERT_ARG(interp) \
+    , PARROT_ASSERT_ARG(seg) \
+    , PARROT_ASSERT_ARG(cursor))
+#define ASSERT_ARGS_Parrot_pf_ConstTable_pack_size \
+     __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
+       PARROT_ASSERT_ARG(interp) \
+    , PARROT_ASSERT_ARG(seg))
+#define ASSERT_ARGS_Parrot_pf_ConstTable_rlookup_num \
+     __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
+       PARROT_ASSERT_ARG(ct))
+#define ASSERT_ARGS_Parrot_pf_ConstTable_rlookup_pmc \
+     __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
+       PARROT_ASSERT_ARG(interp) \
+    , PARROT_ASSERT_ARG(ct) \
+    , PARROT_ASSERT_ARG(v) \
+    , PARROT_ASSERT_ARG(constno) \
+    , PARROT_ASSERT_ARG(idx))
+#define ASSERT_ARGS_Parrot_pf_ConstTable_rlookup_str \
+     __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
+       PARROT_ASSERT_ARG(interp) \
+    , PARROT_ASSERT_ARG(ct) \
+    , PARROT_ASSERT_ARG(s))
+#define ASSERT_ARGS_Parrot_pf_pack __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
+       PARROT_ASSERT_ARG(interp) \
+    , PARROT_ASSERT_ARG(self) \
+    , PARROT_ASSERT_ARG(cursor))
+#define ASSERT_ARGS_Parrot_pf_pack_size __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
+       PARROT_ASSERT_ARG(interp) \
+    , PARROT_ASSERT_ARG(self))
 /* Don't modify between HEADERIZER BEGIN / HEADERIZER END.  Your changes will be lost. */
 /* HEADERIZER END: src/packfile/output.c */
 
@@ -1029,55 +1198,59 @@ size_t PackFile_pack_size(PARROT_INTERP, ARGMOD(PackFile *self))
 /* Don't modify between HEADERIZER BEGIN / HEADERIZER END.  Your changes will be lost. */
 
 PARROT_EXPORT
-PARROT_CANNOT_RETURN_NULL
-PackFile_Segment * PackFile_Annotations_new(PARROT_INTERP)
-        __attribute__nonnull__(1);
-
-PARROT_EXPORT
-void PackFile_ConstTable_clear(PARROT_INTERP,
-    ARGMOD(PackFile_ConstTable *self))
+PARROT_DEPRECATED
+void PackFile_add_segment(PARROT_INTERP,
+    ARGMOD(PackFile_Directory *dir),
+    ARGMOD(PackFile_Segment *seg))
         __attribute__nonnull__(1)
         __attribute__nonnull__(2)
-        FUNC_MODIFIES(*self);
+        __attribute__nonnull__(3)
+        FUNC_MODIFIES(*dir)
+        FUNC_MODIFIES(*seg);
 
 PARROT_EXPORT
 PARROT_WARN_UNUSED_RESULT
 PARROT_CAN_RETURN_NULL
-const opcode_t * PackFile_ConstTable_unpack(PARROT_INTERP,
-    ARGMOD(PackFile_Segment *seg),
-    ARGIN(const opcode_t *cursor))
+PARROT_DEPRECATED
+PackFile_Segment * PackFile_find_segment(PARROT_INTERP,
+    ARGIN_NULLOK(PackFile_Directory *dir),
+    ARGIN(const STRING *name),
+    int sub_dir)
         __attribute__nonnull__(1)
-        __attribute__nonnull__(2)
-        __attribute__nonnull__(3)
-        FUNC_MODIFIES(*seg);
+        __attribute__nonnull__(3);
 
 PARROT_EXPORT
-void PackFile_funcs_register(PARROT_INTERP,
-    ARGMOD(PackFile *pf),
-    UINTVAL type,
-    const PackFile_funcs funcs)
-        __attribute__nonnull__(2)
-        FUNC_MODIFIES(*pf);
+PARROT_DEPRECATED
+INTVAL PackFile_map_segments(PARROT_INTERP,
+    ARGIN(const PackFile_Directory *dir),
+    PackFile_map_segments_func_t callback,
+    ARGIN_NULLOK(void *user_data))
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2);
 
 PARROT_EXPORT
+PARROT_DEPRECATED
 void PackFile_Segment_destroy(PARROT_INTERP, ARGMOD(PackFile_Segment *self))
         __attribute__nonnull__(1)
         __attribute__nonnull__(2)
         FUNC_MODIFIES(*self);
 
 PARROT_EXPORT
+PARROT_DEPRECATED
 void PackFile_Segment_dump(PARROT_INTERP,
     ARGIN(const PackFile_Segment *self))
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
 PARROT_EXPORT
+PARROT_DEPRECATED
 PARROT_WARN_UNUSED_RESULT
 PARROT_CANNOT_RETURN_NULL
 PackFile_Segment * PackFile_Segment_new(PARROT_INTERP)
         __attribute__nonnull__(1);
 
 PARROT_EXPORT
+PARROT_DEPRECATED
 PARROT_WARN_UNUSED_RESULT
 PARROT_CANNOT_RETURN_NULL
 PackFile_Segment * PackFile_Segment_new_seg(PARROT_INTERP,
@@ -1091,6 +1264,7 @@ PackFile_Segment * PackFile_Segment_new_seg(PARROT_INTERP,
         FUNC_MODIFIES(*dir);
 
 PARROT_EXPORT
+PARROT_DEPRECATED
 PARROT_WARN_UNUSED_RESULT
 PARROT_CANNOT_RETURN_NULL
 opcode_t * PackFile_Segment_pack(PARROT_INTERP,
@@ -1101,12 +1275,14 @@ opcode_t * PackFile_Segment_pack(PARROT_INTERP,
         __attribute__nonnull__(3);
 
 PARROT_EXPORT
+PARROT_DEPRECATED
 size_t PackFile_Segment_packed_size(PARROT_INTERP,
     ARGIN(PackFile_Segment *self))
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
 PARROT_EXPORT
+PARROT_DEPRECATED
 PARROT_WARN_UNUSED_RESULT
 PARROT_CAN_RETURN_NULL
 const opcode_t * PackFile_Segment_unpack(PARROT_INTERP,
@@ -1117,62 +1293,101 @@ const opcode_t * PackFile_Segment_unpack(PARROT_INTERP,
         __attribute__nonnull__(3)
         FUNC_MODIFIES(*self);
 
+PARROT_EXPORT
+void Parrot_pf_add_segment(PARROT_INTERP,
+    ARGMOD(PackFile_Directory *dir),
+    ARGMOD(PackFile_Segment *seg))
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
+        __attribute__nonnull__(3)
+        FUNC_MODIFIES(*dir)
+        FUNC_MODIFIES(*seg);
+
+PARROT_EXPORT
+void Parrot_pf_destroy_segment(PARROT_INTERP,
+    ARGMOD(PackFile_Segment *self))
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
+        FUNC_MODIFIES(*self);
+
+PARROT_EXPORT
+void Parrot_pf_dump_segment(PARROT_INTERP,
+    ARGIN(const PackFile_Segment *self))
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2);
+
+PARROT_EXPORT
+PARROT_WARN_UNUSED_RESULT
+PARROT_CAN_RETURN_NULL
+PackFile_Segment * Parrot_pf_find_segment(PARROT_INTERP,
+    ARGIN_NULLOK(PackFile_Directory *dir),
+    ARGIN(const STRING *name),
+    int sub_dir)
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(3);
+
+PARROT_EXPORT
+INTVAL Parrot_pf_map_segments(PARROT_INTERP,
+    ARGIN(const PackFile_Directory *dir),
+    PackFile_map_segments_func_t callback,
+    ARGIN_NULLOK(void *user_data))
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2);
+
+PARROT_EXPORT
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
+PackFile_Segment * Parrot_pf_new_segment(PARROT_INTERP,
+    ARGMOD(PackFile_Directory *dir),
+    UINTVAL type,
+    ARGIN(STRING *name),
+    int add)
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
+        __attribute__nonnull__(4)
+        FUNC_MODIFIES(*dir);
+
 void default_dump_header(PARROT_INTERP, ARGIN(const PackFile_Segment *self))
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
-void PackFile_Annotations_destroy(PARROT_INTERP,
-    ARGMOD(PackFile_Segment *seg))
+void pf_register_standard_funcs(ARGMOD(PackFile *pf))
+        __attribute__nonnull__(1)
+        FUNC_MODIFIES(*pf);
+
+PARROT_WARN_UNUSED_RESULT
+PARROT_CANNOT_RETURN_NULL
+opcode_t * pf_segment_pack(PARROT_INTERP,
+    ARGIN(PackFile_Segment *self),
+    ARGIN(opcode_t *cursor))
         __attribute__nonnull__(1)
         __attribute__nonnull__(2)
-        FUNC_MODIFIES(*seg);
+        __attribute__nonnull__(3);
 
-void PackFile_Annotations_dump(PARROT_INTERP,
-    ARGIN(const PackFile_Segment *seg))
+size_t pf_segment_packed_size(PARROT_INTERP, ARGIN(PackFile_Segment *self))
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
 PARROT_WARN_UNUSED_RESULT
-PARROT_CANNOT_RETURN_NULL
-opcode_t * PackFile_Annotations_pack(PARROT_INTERP,
-    ARGIN(PackFile_Segment *seg),
-    ARGOUT(opcode_t *cursor))
-        __attribute__nonnull__(2)
-        __attribute__nonnull__(3)
-        FUNC_MODIFIES(*cursor);
-
-PARROT_WARN_UNUSED_RESULT
-PARROT_PURE_FUNCTION
-size_t PackFile_Annotations_packed_size(PARROT_INTERP,
-    ARGMOD(PackFile_Segment *seg))
-        __attribute__nonnull__(2)
-        FUNC_MODIFIES(*seg);
-
-PARROT_CANNOT_RETURN_NULL
-const opcode_t * PackFile_Annotations_unpack(PARROT_INTERP,
-    ARGMOD(PackFile_Segment *seg),
+PARROT_CAN_RETURN_NULL
+const opcode_t * pf_segment_unpack(PARROT_INTERP,
+    ARGMOD(PackFile_Segment *self),
     ARGIN(const opcode_t *cursor))
         __attribute__nonnull__(1)
         __attribute__nonnull__(2)
         __attribute__nonnull__(3)
-        FUNC_MODIFIES(*seg);
+        FUNC_MODIFIES(*self);
 
-void pf_register_standard_funcs(PARROT_INTERP, ARGMOD(PackFile *pf))
-        __attribute__nonnull__(1)
-        __attribute__nonnull__(2)
-        FUNC_MODIFIES(*pf);
-
-#define ASSERT_ARGS_PackFile_Annotations_new __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
-       PARROT_ASSERT_ARG(interp))
-#define ASSERT_ARGS_PackFile_ConstTable_clear __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
+#define ASSERT_ARGS_PackFile_add_segment __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp) \
-    , PARROT_ASSERT_ARG(self))
-#define ASSERT_ARGS_PackFile_ConstTable_unpack __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
+    , PARROT_ASSERT_ARG(dir) \
+    , PARROT_ASSERT_ARG(seg))
+#define ASSERT_ARGS_PackFile_find_segment __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp) \
-    , PARROT_ASSERT_ARG(seg) \
-    , PARROT_ASSERT_ARG(cursor))
-#define ASSERT_ARGS_PackFile_funcs_register __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
-       PARROT_ASSERT_ARG(pf))
+    , PARROT_ASSERT_ARG(name))
+#define ASSERT_ARGS_PackFile_map_segments __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
+       PARROT_ASSERT_ARG(interp) \
+    , PARROT_ASSERT_ARG(dir))
 #define ASSERT_ARGS_PackFile_Segment_destroy __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp) \
     , PARROT_ASSERT_ARG(self))
@@ -1196,28 +1411,42 @@ void pf_register_standard_funcs(PARROT_INTERP, ARGMOD(PackFile *pf))
        PARROT_ASSERT_ARG(interp) \
     , PARROT_ASSERT_ARG(self) \
     , PARROT_ASSERT_ARG(cursor))
+#define ASSERT_ARGS_Parrot_pf_add_segment __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
+       PARROT_ASSERT_ARG(interp) \
+    , PARROT_ASSERT_ARG(dir) \
+    , PARROT_ASSERT_ARG(seg))
+#define ASSERT_ARGS_Parrot_pf_destroy_segment __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
+       PARROT_ASSERT_ARG(interp) \
+    , PARROT_ASSERT_ARG(self))
+#define ASSERT_ARGS_Parrot_pf_dump_segment __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
+       PARROT_ASSERT_ARG(interp) \
+    , PARROT_ASSERT_ARG(self))
+#define ASSERT_ARGS_Parrot_pf_find_segment __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
+       PARROT_ASSERT_ARG(interp) \
+    , PARROT_ASSERT_ARG(name))
+#define ASSERT_ARGS_Parrot_pf_map_segments __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
+       PARROT_ASSERT_ARG(interp) \
+    , PARROT_ASSERT_ARG(dir))
+#define ASSERT_ARGS_Parrot_pf_new_segment __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
+       PARROT_ASSERT_ARG(interp) \
+    , PARROT_ASSERT_ARG(dir) \
+    , PARROT_ASSERT_ARG(name))
 #define ASSERT_ARGS_default_dump_header __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp) \
     , PARROT_ASSERT_ARG(self))
-#define ASSERT_ARGS_PackFile_Annotations_destroy __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
-       PARROT_ASSERT_ARG(interp) \
-    , PARROT_ASSERT_ARG(seg))
-#define ASSERT_ARGS_PackFile_Annotations_dump __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
-       PARROT_ASSERT_ARG(interp) \
-    , PARROT_ASSERT_ARG(seg))
-#define ASSERT_ARGS_PackFile_Annotations_pack __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
-       PARROT_ASSERT_ARG(seg) \
-    , PARROT_ASSERT_ARG(cursor))
-#define ASSERT_ARGS_PackFile_Annotations_packed_size \
-     __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
-       PARROT_ASSERT_ARG(seg))
-#define ASSERT_ARGS_PackFile_Annotations_unpack __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
-       PARROT_ASSERT_ARG(interp) \
-    , PARROT_ASSERT_ARG(seg) \
-    , PARROT_ASSERT_ARG(cursor))
 #define ASSERT_ARGS_pf_register_standard_funcs __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
+       PARROT_ASSERT_ARG(pf))
+#define ASSERT_ARGS_pf_segment_pack __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp) \
-    , PARROT_ASSERT_ARG(pf))
+    , PARROT_ASSERT_ARG(self) \
+    , PARROT_ASSERT_ARG(cursor))
+#define ASSERT_ARGS_pf_segment_packed_size __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
+       PARROT_ASSERT_ARG(interp) \
+    , PARROT_ASSERT_ARG(self))
+#define ASSERT_ARGS_pf_segment_unpack __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
+       PARROT_ASSERT_ARG(interp) \
+    , PARROT_ASSERT_ARG(self) \
+    , PARROT_ASSERT_ARG(cursor))
 /* Don't modify between HEADERIZER BEGIN / HEADERIZER END.  Your changes will be lost. */
 /* HEADERIZER END: src/packfile/segments.c */
 
