@@ -53,6 +53,11 @@
 #  include <unistd.h>
 #endif /* PARROT_HAS_HEADER_UNISTD */
 
+#ifdef PARROT_HAS_HEADER_PROCESS
+#  include <process.h>
+#  define getpid _getpid
+#endif /* PARROT_HAS_HEADER_PROCESS */
+
 #ifdef PARROT_HAS_HEADER_SYSMMAN
 #  include <sys/mman.h>
 #  ifndef MAP_FAILED
@@ -90,7 +95,11 @@ typedef jmp_buf Parrot_jump_buff;
 #endif
 
 #define NUM_REGISTERS 32
-#define PARROT_MAGIC 0x13155a1
+
+/* Make 'PARROT_MAGIC', at least, agree with the '*.pbc' files,
+   even though it's unused.
+ */
+#define PARROT_MAGIC  0xfe5042430a1a0a
 
 typedef void STRING_FUNCS;
 typedef struct parrot_interp_t Interp;
@@ -126,7 +135,10 @@ typedef struct parrot_interp_t Interp;
 #  endif /* PTR_SIZE == LONG_SIZE */
 #endif /* PTR_SIZE == INTVAL_SIZE */
 #define PTR2INTVAL(p)    INTVAL2PTR(INTVAL, (p))
-#define PTR2UINTVAL(p)    UINTVAL2PTR(UINTVAL, (p))
+#define PTR2UINTVAL(p)   UINTVAL2PTR(UINTVAL, (p))
+#define PTR2ULONG(p)     UINTVAL2PTR(unsigned long, (p))
+/* cast a const qualifier away. */
+#define PTR_UNCONST(any, p) INTVAL2PTR((any), INTVAL2PTR(INTVAL, (p)))
 
 /*
  * some compilers don't like lvalue casts, so macroize them
@@ -184,15 +196,17 @@ typedef void (*funcptr_t)(void);
 
 /* define macros for converting between data and function pointers.  As it
  * turns out, ANSI C does appear to permit you to do this conversion if you
- * convert the value to an integer (well, a value type large enough to hold
+ * convert the value to a long (well, a value type large enough to hold
  * a pointer) in between.  Believe it or not, this even works on TenDRA (tcc).
- *
- * NOTE!  UINTVAL is incorrect below.  It should be UINTPTR or something like
- * that. The equivalent of C99's uintptr_t- a non-pointer data type that can
- * hold a pointer.
+ * Note that on win64 MSVC a long is 4 bytes, not 8.
  */
-#define D2FPTR(x) UINTVAL2PTR(funcptr_t, PTR2UINTVAL(x))
-#define F2DPTR(x) UINTVAL2PTR(void *, PTR2UINTVAL((funcptr_t) (x)))
+#ifdef _MSC_VER
+#  define D2FPTR(x) UINTVAL2PTR(funcptr_t, PTR2UINTVAL(x))
+#  define F2DPTR(x) UINTVAL2PTR(void *, PTR2UINTVAL((funcptr_t) (x)))
+#else
+#  define D2FPTR(x) UINTVAL2PTR(funcptr_t, PTR2ULONG(x))
+#  define F2DPTR(x) UINTVAL2PTR(void *, PTR2ULONG((funcptr_t) (x)))
+#endif
 
 /* On Win32 we need the constant O_BINARY for open() (at least for Borland C),
    but on UNIX it doesn't exist, so set it to 0 if it's not defined
@@ -204,7 +218,9 @@ typedef void (*funcptr_t)(void);
 /* Hide our struct copying behind macros */
 /* Copying to struct pointer from struct pointer */
 #define STRUCT_COPY(d, s)    (PARROT_ASSERT(d), PARROT_ASSERT(s), *(d)=*(s))
-#define STRUCT_COPY_N(d, s, n) (PARROT_ASSERT(d), PARROT_ASSERT(s), PARROT_ASSERT(sizeof (*(d))==sizeof (*(s))), memcpy((d), (s), sizeof (*(d))*(n)))
+#define STRUCT_COPY_N(d, s, n) \
+    (PARROT_ASSERT(d), PARROT_ASSERT(s), PARROT_ASSERT(sizeof (*(d))==sizeof (*(s))), \
+     memcpy((d), (s), sizeof (*(d))*(n)))
 /* Copying to struct pointer from struct */
 #define STRUCT_COPY_FROM_STRUCT(d, s)    (PARROT_ASSERT(d), *(d)=(s))
 
@@ -266,6 +282,7 @@ typedef struct PackFile_ByteCode PackFile_ByteCode;
 #include "parrot/namespace.h"
 #include "parrot/hll.h"
 #include "parrot/pbcversion.h"
+#include "parrot/disassemble.h"
 
 #endif /* PARROT_PARROT_H_GUARD */
 
