@@ -13,7 +13,6 @@
 
 #include "parrot/core_pmcs.h"
 #include "parrot/pobj.h"
-#include "parrot/thread.h"
 
 #define PARROT_MAX_CLASSES 100
 
@@ -50,6 +49,12 @@ PARROT_EXPORT
 void Parrot_pmc_create_mro(PARROT_INTERP, INTVAL type)
         __attribute__nonnull__(1);
 
+PARROT_EXPORT
+void Parrot_pmc_delprop(PARROT_INTERP, ARGIN(PMC *pmc), ARGIN(STRING *key))
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
+        __attribute__nonnull__(3);
+
 PARROT_EXPORT
 void Parrot_pmc_destroy(PARROT_INTERP, ARGMOD(PMC *pmc))
         __attribute__nonnull__(1)
@@ -76,6 +81,19 @@ PARROT_WARN_UNUSED_RESULT
 INTVAL Parrot_pmc_get_type_str(PARROT_INTERP, ARGIN_NULLOK(STRING *name))
         __attribute__nonnull__(1);
 
+PARROT_EXPORT
+PARROT_CAN_RETURN_NULL
+PMC * Parrot_pmc_getprop(PARROT_INTERP, ARGIN(PMC *pmc), ARGIN(STRING *key))
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
+        __attribute__nonnull__(3);
+
+PARROT_EXPORT
+PARROT_CAN_RETURN_NULL
+PMC * Parrot_pmc_getprops(PARROT_INTERP, ARGIN(PMC *pmc))
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2);
+
 PARROT_EXPORT
 PARROT_PURE_FUNCTION
 PARROT_WARN_UNUSED_RESULT
@@ -88,31 +106,6 @@ PARROT_WARN_UNUSED_RESULT
 PMC * Parrot_pmc_new(PARROT_INTERP, INTVAL base_type)
         __attribute__nonnull__(1);
 
-PARROT_EXPORT
-PARROT_CANNOT_RETURN_NULL
-PMC * Parrot_pmc_new_constant(PARROT_INTERP, INTVAL base_type)
-        __attribute__nonnull__(1);
-
-PARROT_EXPORT
-PARROT_CANNOT_RETURN_NULL
-PMC * Parrot_pmc_new_constant_init(PARROT_INTERP,
-    INTVAL base_type,
-    ARGIN_NULLOK(PMC *init))
-        __attribute__nonnull__(1);
-
-PARROT_EXPORT
-PARROT_CANNOT_RETURN_NULL
-PMC * Parrot_pmc_new_constant_init_int(PARROT_INTERP,
-    INTVAL base_type,
-    INTVAL init)
-        __attribute__nonnull__(1);
-
-PARROT_EXPORT
-PARROT_CANNOT_RETURN_NULL
-PARROT_WARN_UNUSED_RESULT
-PMC * Parrot_pmc_new_constant_noinit(PARROT_INTERP, INTVAL base_type)
-        __attribute__nonnull__(1);
-
 PARROT_EXPORT
 PARROT_CANNOT_RETURN_NULL
 PARROT_WARN_UNUSED_RESULT
@@ -139,6 +132,11 @@ PARROT_CANNOT_RETURN_NULL
 PMC * Parrot_pmc_new_noinit(PARROT_INTERP, INTVAL base_type)
         __attribute__nonnull__(1);
 
+PARROT_EXPORT
+PARROT_PURE_FUNCTION
+PARROT_CAN_RETURN_NULL
+Parrot_PMC Parrot_pmc_null(void);
+
 PARROT_EXPORT
 INTVAL Parrot_pmc_register_new_type(PARROT_INTERP, ARGIN(STRING *name))
         __attribute__nonnull__(1)
@@ -180,6 +178,16 @@ PMC * Parrot_pmc_reuse_init(PARROT_INTERP,
         __attribute__nonnull__(4)
         FUNC_MODIFIES(*pmc);
 
+PARROT_EXPORT
+void Parrot_pmc_setprop(PARROT_INTERP,
+    ARGIN(PMC *pmc),
+    ARGIN(STRING *key),
+    ARGIN(PMC *value))
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
+        __attribute__nonnull__(3)
+        __attribute__nonnull__(4);
+
 void Parrot_pmc_free_temporary(PARROT_INTERP, ARGMOD(PMC *pmc))
         __attribute__nonnull__(1)
         __attribute__nonnull__(2)
@@ -209,6 +217,10 @@ INTVAL Parrot_pmc_type_does(PARROT_INTERP,
        PARROT_ASSERT_ARG(interp))
 #define ASSERT_ARGS_Parrot_pmc_create_mro __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp))
+#define ASSERT_ARGS_Parrot_pmc_delprop __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
+       PARROT_ASSERT_ARG(interp) \
+    , PARROT_ASSERT_ARG(pmc) \
+    , PARROT_ASSERT_ARG(key))
 #define ASSERT_ARGS_Parrot_pmc_destroy __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp) \
     , PARROT_ASSERT_ARG(pmc))
@@ -223,19 +235,16 @@ INTVAL Parrot_pmc_type_does(PARROT_INTERP,
     , PARROT_ASSERT_ARG(name))
 #define ASSERT_ARGS_Parrot_pmc_get_type_str __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp))
+#define ASSERT_ARGS_Parrot_pmc_getprop __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
+       PARROT_ASSERT_ARG(interp) \
+    , PARROT_ASSERT_ARG(pmc) \
+    , PARROT_ASSERT_ARG(key))
+#define ASSERT_ARGS_Parrot_pmc_getprops __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
+       PARROT_ASSERT_ARG(interp) \
+    , PARROT_ASSERT_ARG(pmc))
 #define ASSERT_ARGS_Parrot_pmc_is_null __attribute__unused__ int _ASSERT_ARGS_CHECK = (0)
 #define ASSERT_ARGS_Parrot_pmc_new __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp))
-#define ASSERT_ARGS_Parrot_pmc_new_constant __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
-       PARROT_ASSERT_ARG(interp))
-#define ASSERT_ARGS_Parrot_pmc_new_constant_init __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
-       PARROT_ASSERT_ARG(interp))
-#define ASSERT_ARGS_Parrot_pmc_new_constant_init_int \
-     __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
-       PARROT_ASSERT_ARG(interp))
-#define ASSERT_ARGS_Parrot_pmc_new_constant_noinit \
-     __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
-       PARROT_ASSERT_ARG(interp))
 #define ASSERT_ARGS_Parrot_pmc_new_from_type __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp) \
     , PARROT_ASSERT_ARG(key))
@@ -246,6 +255,7 @@ INTVAL Parrot_pmc_type_does(PARROT_INTERP,
        PARROT_ASSERT_ARG(interp))
 #define ASSERT_ARGS_Parrot_pmc_new_noinit __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp))
+#define ASSERT_ARGS_Parrot_pmc_null __attribute__unused__ int _ASSERT_ARGS_CHECK = (0)
 #define ASSERT_ARGS_Parrot_pmc_register_new_type __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp) \
     , PARROT_ASSERT_ARG(name))
@@ -260,6 +270,11 @@ INTVAL Parrot_pmc_type_does(PARROT_INTERP,
        PARROT_ASSERT_ARG(interp) \
     , PARROT_ASSERT_ARG(pmc) \
     , PARROT_ASSERT_ARG(init))
+#define ASSERT_ARGS_Parrot_pmc_setprop __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
+       PARROT_ASSERT_ARG(interp) \
+    , PARROT_ASSERT_ARG(pmc) \
+    , PARROT_ASSERT_ARG(key) \
+    , PARROT_ASSERT_ARG(value))
 #define ASSERT_ARGS_Parrot_pmc_free_temporary __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp) \
     , PARROT_ASSERT_ARG(pmc))
@@ -274,24 +289,6 @@ INTVAL Parrot_pmc_type_does(PARROT_INTERP,
 /* Don't modify between HEADERIZER BEGIN / HEADERIZER END.  Your changes will be lost. */
 /* HEADERIZER END: src/pmc.c */
 
-/* Allows compatibility with old name (see r44190) */
-#define Parrot_create_mro       Parrot_pmc_create_mro
-#define PMC_is_null             Parrot_pmc_is_null
-#define constant_pmc_new        Parrot_pmc_new_constant
-#define constant_pmc_new_init   Parrot_pmc_new_constant_init
-#define gc_register_pmc         Parrot_pmc_gc_register
-#define gc_unregister_pmc       Parrot_pmc_gc_unregister
-#define pmc_new                 Parrot_pmc_new
-#define pmc_new_init            Parrot_pmc_new_init
-#define pmc_new_noinit          Parrot_pmc_new_noinit
-#define pmc_register            Parrot_pmc_register_new_type
-#define pmc_reuse               Parrot_pmc_reuse
-#define pmc_reuse_by_class      Parrot_pmc_reuse_by_class
-#define pmc_reuse_init          Parrot_pmc_reuse_init
-#define pmc_reuse_no_init       Parrot_pmc_reuse_noinit
-#define pmc_type                Parrot_pmc_get_type_str
-#define pmc_type_p              Parrot_pmc_get_type
-
 #define PMC_IS_TYPE(p, t) ((p)->vtable->base_type == enum_class_ ## t)
 #define PMC_IS_TYPE_ENUM(p, e) ((p)->vtable->base_type == (e))
 
