@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2011, Parrot Foundation.
+# Copyright (C) 2007-2014, Parrot Foundation.
 package Parrot::Configure::Options::Conf;
 
 use strict;
@@ -51,12 +51,13 @@ General Options:
 
 Compile Options:
 
-   --debugging=0        Disable debugging, default = 1
+   --debugging=-g3      Enable and set cc debugging flag, default -g
    --inline             Compiler supports inline
    --optimize           Optimized compile
    --optimize=flags     Add given optimizer flags
-   --parrot_is_shared   Link parrot dynamically
-   --disable-rpath      Link without rpath (user must set LD_LIBRARY_PATH)
+   --{en,dis}able-shared  How to link libparrot
+   --{en,dis}able-static  How to link libparrot
+   --{en,dis}able-rpath   If without rpath (user must set LD_LIBRARY_PATH)
    --m=32               Build 32bit executable on 64-bit architecture.
    --profile            Turn on profiled compile (gcc only for now)
    --cage               [CAGE] compile includes many additional warnings
@@ -77,7 +78,7 @@ Compile Options:
 
    --no-line-directives Disable creation of C #line directives
    --define=inet_aton   Quick hack to use inet_aton instead of inet_pton
-   --gc=(type)          Which implementation of GC to use. One of ms, ms2 or gms.
+   --gc=(type)          Which implementation of GC to use. One of gms, ms, ms2 or inf.
 
 Parrot Options:
 
@@ -86,7 +87,7 @@ Parrot Options:
    --opcode=(type)      Use the given type for opcodes
    --ops=(files)        Use the given ops files
 
-   --without-threads    Build parrot without thread support
+   --without-threads    Build parrot without OS thread support
    --without-core-nci-thunks
                         Build parrot without core-required
                         statically compiled NCI call frames
@@ -95,7 +96,7 @@ Parrot Options:
                         Build parrot without unnecessary
                         statically compiled NCI call frames
 
-External Library Options:
+External Library Options (use --with- or --without-):
 
    --with-llvm          Link to LLVM if it is available
    --without-gettext    Build parrot without gettext support
