@@ -1,4 +1,4 @@
-# Copyright (C) 2004-2012, Parrot Foundation.
+# Copyright (C) 2004-2014, Parrot Foundation.
 
 =head1 NAME
 
@@ -46,7 +46,7 @@ looking for the distribution directory, and returns it if it finds it.
 The search is only performed once.
 
 The criterion is that there should be a F<README.pod> file beginning with
-the words "# Copyright (C) 2001-2012, Parrot Foundation." in the directory.
+the words "# Copyright (C) 2001-2014, Parrot Foundation." in the directory.
 
 Raises an exception if the distribution root is not found.
 
@@ -73,7 +73,7 @@ Raises an exception if the distribution root is not found.
 
         while ( $self = $self->SUPER::new($path) ) {
             if (    $self->file_exists_with_name($file)
-                and $self->file_with_name($file)->read =~ m/^\# Copyright \(C\) 2001-2012, Parrot Foundation\./s )
+                and $self->file_with_name($file)->read =~ m/^\# Copyright \(C\) 2001-2014, Parrot Foundation\./s )
             {
                 $dist = $self;
                 last;
@@ -690,8 +690,8 @@ sub perl_module_file_directories {
     return
         map $self->directory_with_name($_) =>
         map( "config/$_" => qw<auto auto/cpu/i386 auto/cpu/ppc
-            auto/cpu/sun4 auto/cpu/x86_64
-            gen gen/cpu/i386 gen/cpu/x86_64 init init/hints inter> ),
+            auto/cpu/sun4 auto/cpu/amd64
+            gen gen/cpu/i386 gen/cpu/amd64 init init/hints inter> ),
         'ext/Parrot-Embed/lib/Parrot',
         map( "lib/$_" => qw<
             Class Digest/Perl File Parrot Parse Pod Pod/Simple Test Text
@@ -821,6 +821,10 @@ sub slurp {
     return $buf;
 }
 
+=back
+
+=cut
+
 1;
 
 # Local Variables:
