@@ -1,9 +1,9 @@
 package Parrot::Install;
-# Copyright (C) 2001-2009, Parrot Foundation.
+# Copyright (C) 2001-2014, Parrot Foundation.
 use strict;
 use warnings;
-use File::Basename qw(dirname);
-use File::Copy;
+use File::Basename qw( dirname );
+use File::Copy qw( cp );
 use File::Path qw( mkpath );
 use File::Spec;
 use base qw( Exporter );
@@ -12,6 +12,9 @@ our @EXPORT_OK = qw(
     create_directories
     install_files
 );
+use lib 'lib';
+use Parrot::Config;
+use Parrot::BuildUtil;
 
 #################### DOCUMENTATION ####################
 
@@ -29,10 +32,10 @@ Parrot::Install - Functionality for installation programs
 
 =head1 DESCRIPTION
 
-This module exports on demand only three subroutines used in the Parrot
+This module exports on demand some subroutines used in the Parrot
 installation programs F<tools/dev/install_files.pl> and
-F<tools/dev/install_dev_files.pl>.  The subroutines are tested by tests found
-in F<t/tools/install/>.
+F<tools/dev/install_dev_files.pl>.
+The subroutines are tested by tests found in F<t/tools/install/>.
 
 =head1 SUBROUTINES
 
@@ -129,7 +132,7 @@ sub lines_to_files {
                 $filehash = &{ $thash->{transform} }($filehash);
                 ref($filehash) eq 'HASH' or die "Error: transform didn't return a hash for key '$tkey'\n";
                 $filehash->{Dest} = File::Spec->catdir(
-                    $options_ref->{$thash->{optiondir} . 'dir'},
+                    $options_ref->{$thash->{optiondir} . 'dir'} || '',
                     @{ $filehash->{DestDirs} },
                     $filehash->{Dest}
                 );
@@ -185,6 +188,86 @@ sub create_directories {
     return 1;
 }
 
+=head2 C<sanitycheck_install()>
+
+Check if the generated F<MANIFEST.generated> contains some typically
+needed files. Dies if not ok.
+
+If you update your repo and rebuild parrot without perl Configure.PL
+which deletes MANIFEST.generated anew, MANIFEST.generated might not
+contain already up-to-date targets, thus not creating the required
+MANIFEST.generated lines.
+
+TODO: Keep a manual list of all to be installed files, merge it with
+MANIFEST.generated and die if a file in the merged list does not exist,
+i.e. cannot be installed.
+
+=cut
+
+sub _check_manifest {
+    my ($file,$gen) = @_;
+    my $manifest = $_[2];
+    my $fname = "MANIFEST.generated";
+    unless ($manifest =~ /^\Q$file\E/m) {
+        if ($^O eq 'MSWin32') { # Windows has mixed paths
+            $file =~ s{/}{\\}g;
+            return if $manifest =~ /^\Q$file\E/m;
+        }
+        if ($gen)  {
+            die "Error: configure generated $file missing in $fname.\n"
+              . "make reconfig before make install.\n";
+        }
+        else {
+            die "Error: make generated $file missing in $fname.\n"
+              . "make clean before make install.\n";
+        }
+    }
+}
+
+sub sanitycheck_install {
+    my $fname = "MANIFEST.generated";
+    my $manifest = Parrot::BuildUtil::slurp_file($fname);
+    die "$fname not found.\n" unless $manifest;
+    # configure generated
+    _check_manifest($_, 1, $manifest) for qw(
+        lib/Parrot/Pmc2c/PCCMETHOD_BITS.pm
+        include/parrot/config.h
+        include/parrot/pbcversion.h
+        include/parrot/vtable.h
+        include/parrot/core_pmcs.h
+        lib/Parrot/PMC.pm
+        myconfig
+        lib/Parrot/Config/Generated.pm
+        runtime/parrot/library/config.pir
+    );
+    # make generated
+    my @MAKE_gen = qw(
+        vtable.dump
+        include/parrot/extend_vtable.h
+        runtime/parrot/include/datatypes.pasm
+        runtime/parrot/library/Test/More.pbc
+        compilers/tge/TGE/Parser.pir
+        runtime/parrot/library/P6Regex.pbc
+        runtime/parrot/languages/winxed/winxed.pbc
+        runtime/parrot/library/nqp-setting.pbc
+        runtime/parrot/library/config.pbc
+        runtime/parrot/library/opsc.pbc
+        install_config.fpmc
+        runtime/parrot/library/pcre.pbc
+        runtime/parrot/library/PCT/Grammar.pbc
+    );
+    my @dynext = qw(dynlexpad os file rational subproxy);
+    push @dynext, map {$_."_ops"} qw(bit io sys obscure math trans debug);
+    push @dynext, "select" if $^O !~ /^(MSWin32|msys)$/;
+    push @dynext, "gziphandle" if $PConfig{has_zlib};
+    push @MAKE_gen, "src/install_config".$PConfig{o};
+    push @MAKE_gen, map { "runtime/parrot/dynext/".$_.$PConfig{load_ext} } @dynext;
+    push @MAKE_gen, map { "installable_".$_.$PConfig{exe} }
+      qw(parrot pbc_dump pbc_disassemble parrot_config pbc_merge
+         pbc_to_exe parrot_nci_thunk_gen parrot-ops2c winxed);
+    _check_manifest($_, 0, $manifest) for @MAKE_gen;
+}
+
 =head2 C<install_files()>
 
 B<Purpose:> Install the mentioned files into the appropriate locations.
@@ -200,24 +283,36 @@ list consisting of hashes.
 
 B<Return Value:>  True value.
 
-B<Comment:>
+B<Comment:> Adds all installed files to datadir/parrotdir/MANIFEST
 
 =cut
 
 sub install_files {
-    my($destdir, $dryrun, $files) = @_;
-    my($src, $dest, $mode);
+    my ($options, $type, $files) = @_;
+    ref($options) eq 'HASH' or die "Error: parameter \$options must be a hashref\n";
 
-    ref($files) eq 'ARRAY' or die "Error: parameter \$files must be an array\n";
+    my ($destdir, $datadir, $versiondir, $dryrun) =
+      @$options{qw(destdir datadir versiondir dryrun)};
+    ref($files) eq 'ARRAY' or die "Error: parameter \$files must be an arrayref\n";
+
+    my ($src, $dest, $mdest, $mode, $manifest);
     print("Installing ...\n");
+    if (!$dryrun) {
+        my $destdatadir = $destdir
+          ? File::Spec->catdir( $destdir, $datadir, $versiondir)
+          : File::Spec->catdir( $datadir, $versiondir);
+        my $fname = File::Spec->catdir( $destdatadir, "MANIFEST" . ($type ? ".".$type : '') );
+        open $manifest, ">", $fname or die "Could not create $fname\n";
+        print "$fname\n";
+    }
     foreach my $el ( @$files ) {
         unless(ref($el) eq 'HASH') {
             my($ref) = ref($el);
             warn "Bad reference passed in \$files (want a HASH, got a '$ref')\n";
             next;
         }
-        ( $src, $dest ) = map { $el->{$_} } qw(Source Dest);
-        $dest = $destdir . $dest;
+        ( $src, $mdest ) = map { $el->{$_} } qw(Source Dest);
+        $dest = $destdir . $mdest;
         if ( $dryrun ) {
             print "$src -> $dest\n";
             next;
@@ -237,16 +332,19 @@ sub install_files {
                     # this loop
                     if (-e $dest) {
                         print "$dest\n";
+                        print $manifest "$mdest\n" unless $dryrun;
                         next;
                     }
                 }
             }
-            copy( $src, $dest ) or die "Error: couldn't copy $src to $dest: $!\n";
+            cp( $src, $dest ) or die "Error: couldn't copy $src to $dest: $!\n";
             print "$dest\n";
+            print $manifest "$mdest\n" unless $dryrun;
         }
         $mode = ( stat($src) )[2];
         chmod $mode, $dest;
     }
+    close $manifest unless $dryrun;
     return 1;
 }
 
