@@ -1,4 +1,4 @@
-# Copyright (C) 2004-2011, Parrot Foundation.
+# Copyright (C) 2004-2014, Parrot Foundation.
 
 =head1 NAME
 
@@ -34,6 +34,7 @@ use Parrot::Pmc2c::UtilFunctions qw(
     gen_multi_name
 );
 use Parrot::Pmc2c::PMC::RO ();
+use Parrot::BuildUtil;
 
 sub create {
     my ( $this, $pmc_classname ) = @_;
@@ -70,8 +71,9 @@ sub dump {
 
     # gen_parent_lookup_info( $self, $pmc2cMain, $pmcs );
     # gen_parent_reverse_lookup_info( $self, $pmcs, $vtable_dump );
-
-    Storable::store( $self, $self->filename('.dump') );
+    my $filename = $self->filename('.dump');
+    Storable::nstore( $self, $filename );
+    # add_to_generated( $filename, "[devel]", "src") unless $self->is_dynamic;
 }
 
 # methods
@@ -130,7 +132,7 @@ sub add_mixin {
 
 sub add_attribute {
     my ( $self, $attribute ) = @_;
-    $self->{has_attribute}{ $attribute->name } = @{ $self->{attributes} };
+    $self->{has_attribute}{ $attribute->{name} } = @{ $self->{attributes} };
     push @{ $self->{attributes} }, $attribute;
 }
 
@@ -472,17 +474,21 @@ sub prep_for_emit {
 
 sub generate {
     my ($self) = @_;
-    my $emitter = $self->{emitter} =
-        Parrot::Pmc2c::Emitter->new( $self->filename(".c") );
 
+    my $c_file = $self->filename(".c");
+    my $c_emitter = $self->{emitter} =
+        Parrot::Pmc2c::Emitter->new( $c_file );
     $self->generate_c_file;
-    $emitter->write_to_file;
-
-    $emitter = $self->{emitter} =
-        Parrot::Pmc2c::Emitter->new( $self->filename(".h", $self->is_dynamic) );
+    $c_emitter->write_to_file;
+    # add_to_generated($c_file, "[]", "");
 
+    my $h_file = $self->filename(".h", $self->is_dynamic);
+    my $h_emitter = $self->{emitter} =
+        Parrot::Pmc2c::Emitter->new( $h_file );
     $self->generate_h_file;
-    $emitter->write_to_file;
+    $h_emitter->write_to_file;
+    #add_to_generated($h_file, "[devel]", "include")
+    #  unless $self->is_dynamic and $self->name =~ /^(foo|foo2|rotest|pccmethod_test)$/;
 }
 
 =over 4
@@ -499,7 +505,9 @@ sub generate_c_file {
 
     $c->emit( dont_edit( $self->filename ) );
     if ($self->is_dynamic) {
+        my $uc_name = uc $self->name;
         $c->emit("#define PARROT_IN_EXTENSION\n");
+        $c->emit("#define PARROT_DYNPMC_CLASS_LOAD\n");
         $c->emit("#define CONST_STRING(i, s) Parrot_str_new_constant((i), s)\n");
         $c->emit("#define CONST_STRING_GEN(i, s) Parrot_str_new_constant((i), s)\n");
     }
@@ -509,10 +517,11 @@ sub generate_c_file {
     # The PCC code needs Continuation-related macros from these headers.
     $c->emit("#include \"pmc_continuation.h\"\n");
     $c->emit("#include \"pmc_callcontext.h\"\n");
+    $c->emit("#undef PARROT_DYNPMC_CLASS_LOAD\n") if $self->is_dynamic;
 
     $c->emit( $self->preamble );
 
-    $c->emit( $self->hdecls );
+    $c->emit( $self->hdecls ) unless $self->name eq 'CallContext';
     $c->emit( $self->{ro}->hdecls ) if ( $self->{ro} );
     $self->gen_methods;
 
@@ -545,6 +554,7 @@ sub generate_h_file {
     my ($self)  = @_;
     my $h       = $self->{emitter};
     my $uc_name = uc $self->name;
+    my $lc_name = lc $self->name;
     my $name    = $self->name;
 
     $h->emit( dont_edit( $self->filename ) );
@@ -577,8 +587,23 @@ EOH
     $h->emit("${export}PMC*    Parrot_${name}_get_mro(PARROT_INTERP, ARGMOD(PMC* mro));\n");
     $h->emit("${export}Hash*   Parrot_${name}_get_isa(PARROT_INTERP, ARGMOD_NULLOK(Hash* isa));\n");
 
-
     $self->gen_attributes;
+
+    if ($name eq 'CallContext') {
+        $h->emit( $self->hdecls );
+    }
+
+    if ($self->is_dynamic) {
+        $h->emit(<<"EOH");
+
+${export}Parrot_PMC Parrot_lib_${lc_name}_load(PARROT_INTERP);
+
+#ifndef PARROT_DYNPMC_CLASS_LOAD
+PARROT_DATA INTVAL dynpmc_class_${name};
+#endif
+EOH
+    }
+
     $h->emit(<<"EOH");
 
 #endif /* PARROT_PMC_${uc_name}_H_GUARD */
@@ -598,9 +623,9 @@ in the PMC's C header file.
 sub hdecls {
     my ($self) = @_;
 
-    my $hout;
-    my $name    = $self->name;
-    my $lc_name = $self->name;
+    my $hout = '';
+    my $name = $self->name;
+    my $lc_name = lc($name);
 
     # generate decls for all vtables in this PMC
     foreach my $vt_method_name ( @{ $self->vtable->names } ) {
@@ -616,23 +641,7 @@ sub hdecls {
         $hout .= $method->generate_headers($self);
     }
 
-    my $export = $self->is_dynamic ? 'PARROT_DYNEXT_EXPORT ' : '';
-
-    # class init decl
-    $hout .= "${export}void    Parrot_${name}_class_init(PARROT_INTERP, int, int);\n";
-
-    $export = $self->is_dynamic ? 'PARROT_DYNEXT_EXPORT ' : 'PARROT_EXPORT ';
-
-    $hout .= "${export}VTABLE* Parrot_${lc_name}_update_vtable(ARGMOD(VTABLE*));\n"
-        unless $name eq 'default';
-
-    $hout .= "${export}VTABLE* Parrot_${lc_name}_get_vtable(PARROT_INTERP);\n";
-
-    $hout .= "${export}VTABLE* Parrot_${lc_name}_get_vtable_pointer(PARROT_INTERP);\n"
-        if ($self->is_dynamic);
-
     $self->{hdecls} .= $hout;
-
     return $self->{hdecls};
 }
 
@@ -721,34 +730,20 @@ sub post_method_gen {
         next unless $self->vtable_method_does_write($name);
 
         # Skip methods with manual WBs.
+        next if $method->vtable_method_has_manual_wb;
         next if $self->vtable_method_has_manual_wb($name);
 
+        # Skip unimplemented methods
+        next if $self->unimplemented_vtable($name);
+
+        # Skip for Proxy and Null, they just raise an exception
+        next if $self->name =~ /^Null|Proxy/;
+
         $method = $self->get_method($name);
 
-        #warn "Rewriting " . $self->name . "." . $name;
-        $self->add_method(
-            $method->clone({
-                name => $name.'_orig',
-                type => "ORIG",
-            })
-        );
-
-        # Rewrite body with write barrier.
-        my $body;
-        my $need_result = $method->return_type && $method->return_type !~ 'void';
-
-        $body .= $method->return_type . " result;\nresult = " if $need_result;
-
-        # Get parameters.      strip type from param
-        my $parameters = join ', ',
-                         'INTERP', 'SELF', map { /\s*\*?(\S+)$/; $1 }
-                         split (/,/, $method->parameters);
-        $body .= $method->full_method_name($self->name) . "_orig($parameters);\n";
-
-        $body .= "PARROT_GC_WRITE_BARRIER(interp, _self);\n";
-        $body .= "return result;" if $need_result;
-
-        $method->body(Parrot::Pmc2c::Emitter->text($body) );
+        # Rewrite RETURNs or add simple write barrier to body
+        $method->{need_write_barrier} = 1;
+        $method->body->add_write_barrier($method, $self);
     }
 
     # generate PCC-variants for multis
@@ -794,6 +789,7 @@ EOC
         $self->add_method($new_method);
     }
 }
+
 =item C<gen_methods()>
 
 Returns the C code for the pmc methods.
@@ -828,20 +824,19 @@ Returns the C code for the attribute struct definition.
 
 sub gen_attributes {
     my ($self)     = @_;
-    my $attributes = $self->attributes;
 
-    if ( @{$attributes} ) {
+    if ( @{$self->attributes} ) {
 
-        Parrot::Pmc2c::Attribute::generate_start( $attributes->[0], $self );
+        $self->generate_start();
 
-        foreach my $attribute ( @{$attributes} ) {
-            $attribute->generate_declaration($self);
+        foreach my $attr ( @{$self->attributes} ) {
+            $self->generate_declaration($attr);
         }
 
-        Parrot::Pmc2c::Attribute::generate_end( $attributes->[0], $self );
+        $self->generate_end();
 
-        foreach my $attribute ( @{$attributes} ) {
-            $attribute->generate_accessor($self);
+        foreach my $attr ( @{$self->attributes} ) {
+            $self->generate_accessor($attr);
         }
     }
 }
@@ -963,7 +958,7 @@ sub pmc_class_init_func {
 
         # fix indenting
         $class_init_code =~ s/^/    /mg;
-        $class_init_code = <<ENDOFCODE
+        $class_init_code = <<"ENDOFCODE";
 static void thispmc_class_init(PARROT_INTERP, int entry)
 {
 $class_init_code
@@ -1103,23 +1098,23 @@ EOC
         vt->isa_hash     = Parrot_${classname}_get_isa(interp, NULL);
 EOC
 
-    for my $k ( keys %extra_vt ) {
+    for my $k ( sort keys %extra_vt ) {
         my $k_flags = $self->$k->vtable_flags;
+        my $var = "vt_$k";
         $cout .= <<"EOC";
         {
-            VTABLE                   *vt_$k;
-            vt_${k}                 = Parrot_${classname}_${k}_get_vtable(interp);
-            vt_${k}->base_type      = $enum_name;
-            vt_${k}->flags          = $k_flags;
+            VTABLE * const $var         = Parrot_${classname}_${k}_get_vtable(interp);
+            ${var}->base_type           = $enum_name;
+            ${var}->flags               = $k_flags;
 
-            vt_${k}->attribute_defs = attr_defs;
+            ${var}->attribute_defs      = attr_defs;
 
-            vt_${k}->base_type           = entry;
-            vt_${k}->whoami              = vt->whoami;
-            vt_${k}->provides_str        = vt->provides_str;
-            vt->${k}_variant_vtable      = vt_${k};
-            vt_${k}->${k}_variant_vtable = vt;
-            vt_${k}->isa_hash            = vt->isa_hash;
+            ${var}->base_type           = entry;
+            ${var}->whoami              = vt->whoami;
+            ${var}->provides_str        = vt->provides_str;
+            vt->${k}_variant_vtable     = ${var};
+            ${var}->${k}_variant_vtable = vt;
+            ${var}->isa_hash            = vt->isa_hash;
         }
 
 EOC
@@ -1177,14 +1172,14 @@ EOC
         {
             STRING * const method_name = CONST_STRING_GEN(interp, "$symbol_name");
             STRING * const signature   = CONST_STRING_GEN(interp, "$pcc_signature");
-            register_native_pcc_method_in_ns(interp, entry,
+            Parrot_interp_register_native_pcc_method_in_ns(interp, entry,
                 F2DPTR(Parrot_${classname}_${method_name}),
                 method_name, signature);
         }
 EOC
         if ( $method->{attrs}{write} ) {
             $cout .= <<"EOC";
-        Parrot_mark_method_writes(interp, entry, "$symbol_name");
+        Parrot_interp_mark_method_writes(interp, entry, "$symbol_name");
 EOC
         }
     }
@@ -1216,6 +1211,7 @@ EOC
         }
     } /* pass */
 } /* Parrot_${classname}_class_init */
+
 EOC
 
     if ( $self->is_dynamic ) {
@@ -1344,9 +1340,7 @@ PMC* Parrot_${classname}_get_mro(PARROT_INTERP, ARGMOD(PMC* mro)) {
         mro = Parrot_pmc_new(interp, enum_class_ResizableStringArray);
     }
 $get_mro
-    VTABLE_unshift_string(interp, mro,
-        Parrot_str_new_init(interp, "$classname", @{[length($classname)]},
-            Parrot_default_encoding_ptr, 0));
+    VTABLE_unshift_string(interp, mro, CONST_STRING_GEN(interp, "$classname"));
     return mro;
 }
 
@@ -1574,7 +1568,7 @@ sub generate_single_case {
     if ($type eq 'DEFAULT' || $type eq 'PMC') {
         # For default case we have to handle return manually.
         my ($pcc_signature, $retval, $call_tail, $pcc_return)
-                = gen_defaul_case_wrapping($impl);
+                = gen_default_case_wrapping($impl);
         my $dispatch = "Parrot_mmd_multi_dispatch_from_c_args(INTERP, \"$vt_method_name\", \"$pcc_signature\", SELF, $parameters$call_tail);";
 
         $case = <<"CASE";
@@ -1614,7 +1608,7 @@ CASE
 
 # Generate (pcc_signature, retval holder, pcc_call_tail, return statement)
 # for default case in switch.
-sub gen_defaul_case_wrapping {
+sub gen_default_case_wrapping {
     my $method = shift;
 
     local $_ = $method->return_type;
@@ -1655,6 +1649,239 @@ sub gen_defaul_case_wrapping {
     }
 }
 
+=head2 C<generate_start>
+
+Generate and emit the C code for the start of an attribute struct.
+
+=cut
+
+sub generate_start {
+    my ( $pmc ) = @_;
+
+    $pmc->{emitter}->emit(<<"EOH");
+
+/* $pmc->{name} PMC's underlying struct. */
+typedef struct Parrot_$pmc->{name}_attributes {
+EOH
+
+    return 1;
+}
+
+=head2 C<generate_end>
+
+Generate and emit the C code for the end of an attribute struct.
+
+=cut
+
+sub generate_end {
+    my ( $pmc ) = @_;
+    my $name           = $pmc->{name};
+    my $ucname         = uc($name);
+
+    $pmc->{emitter}->emit(<<"EOH");
+} Parrot_${name}_attributes;
+
+/* Macro to access underlying structure of a $name PMC. */
+#define PARROT_${ucname}(o) ((Parrot_${name}_attributes *) PMC_data(o))
+
+EOH
+
+    return 1;
+}
+
+=head2 C<generate_declaration>
+
+Generate and emit the C code for an attribute declaration.
+
+=cut
+
+sub generate_declaration {
+    my ( $pmc, $attribute ) = @_;
+    my $decl = '    ' . $attribute->{type} . ' ' . $attribute->{name} . $attribute->{array_size} . ";\n";
+
+    $pmc->{emitter}->emit($decl);
+
+    return 1;
+}
+
+=head2 C<generate_accessor>
+
+Generate and emit the C code for an attribute get/set accessor pair.
+
+=cut
+
+sub generate_accessor {
+    my ( $pmc, $attribute ) = @_;
+
+    my $pmcname        = $pmc->{name};
+    my $attrtype       = $attribute->{type};
+    my $attrname       = $attribute->{name};
+    my $isfuncptr      = 0;
+    my $origtype       = $attrtype;
+    if($attrname =~ m/\(\*(\w*)\)\((.*?)\)/) {
+        $isfuncptr = 1;
+        $origtype = $attrtype . " (*)(" . $2 . ")";
+        $attrname = $1;
+    }
+
+    # Store regexes used to check some types to avoid repetitions
+    my $isptrtostring = qr/STRING\s*\*$/;
+    my $isptrtopmc    = qr/PMC\s*\*$/;
+
+    my $inherit        = 1;
+    my $decl           = <<"EOA";
+
+/* Generated macro accessors for '$attrname' attribute of $pmcname PMC. */
+#define GETATTR_${pmcname}_${attrname}(interp, pmc, dest) \\
+EOA
+
+    # Nobody derives from CallContext, the arg is always proper, and we need the speed
+    if ($pmcname eq "CallContext") {
+        $decl .= <<"EOA";
+    (dest) = PARROT_CALLCONTEXT(pmc)->${attrname}
+
+#define SETATTR_${pmcname}_${attrname}(interp, pmc, value) \\
+    PARROT_CALLCONTEXT(pmc)->${attrname} = (value)
+EOA
+    }
+
+    else {
+        $decl .= <<"EOA";
+    do { \\
+        if (!PObj_is_object_TEST(pmc)) { \\
+            (dest) = ((Parrot_${pmcname}_attributes *)PMC_data(pmc))->$attrname; \\
+        } \\
+        else { \\
+EOA
+
+        if ($isfuncptr == 1) {
+            $decl .= <<"EOA";
+            Parrot_ex_throw_from_c_noargs(interp, EXCEPTION_INVALID_OPERATION, \\
+                "Attributes of type '$origtype' cannot be " \\
+                "subclassed from a high-level PMC."); \\
+EOA
+        }
+        elsif ($attrtype eq "INTVAL") {
+            $decl .= <<"EOA";
+            PMC * const attr_value = VTABLE_get_attr_str(interp, \\
+                              pmc, Parrot_str_new_constant(interp, "$attrname")); \\
+            (dest) = (PMC_IS_NULL(attr_value) ? (INTVAL) 0: VTABLE_get_integer(interp, attr_value)); \\
+EOA
+        }
+        elsif ($attrtype eq "FLOATVAL") {
+            $decl .= <<"EOA";
+            PMC * const attr_value = VTABLE_get_attr_str(interp, \\
+                              pmc, Parrot_str_new_constant(interp, "$attrname")); \\
+            (dest) =  (PMC_IS_NULL(attr_value) ? (FLOATVAL) 0.0: VTABLE_get_number(interp, attr_value)); \\
+EOA
+        }
+        elsif ($attrtype =~ $isptrtostring) {
+            $decl .= <<"EOA";
+            PMC * const attr_value = VTABLE_get_attr_str(interp, \\
+                              pmc, Parrot_str_new_constant(interp, "$attrname")); \\
+            (dest) =  (PMC_IS_NULL(attr_value) ? (STRING *)NULL : VTABLE_get_string(interp, attr_value)); \\
+EOA
+        }
+        elsif ($attrtype =~ $isptrtopmc) {
+            $decl .= <<"EOA";
+            (dest) = VTABLE_get_attr_str(interp, \\
+                              pmc, Parrot_str_new_constant(interp, "$attrname")); \\
+EOA
+        }
+
+        else {
+            $inherit = 0;
+            $decl .= <<"EOA";
+            Parrot_ex_throw_from_c_noargs(interp, EXCEPTION_INVALID_OPERATION, \\
+                "Attributes of type '$attrtype' cannot be " \\
+                "subclassed from a high-level PMC."); \\
+EOA
+        }
+
+        $decl .= <<"EOA";
+        } \\
+    } while (0)
+EOA
+
+        $decl .= <<"EOA";
+#define SETATTR_${pmcname}_${attrname}(interp, pmc, value) \\
+    do { \\
+        if (PObj_is_object_TEST(pmc)) { \\
+EOA
+
+        if ($isfuncptr == 1) {
+            $decl .= <<"EOA";
+            Parrot_ex_throw_from_c_noargs(interp, EXCEPTION_INVALID_OPERATION, \\
+                "Attributes of type '$origtype' cannot be " \\
+                "subclassed from a high-level PMC."); \\
+EOA
+        }
+        elsif ($attrtype eq "INTVAL") {
+            $decl .= <<"EOA";
+            PMC * const attr_value = Parrot_pmc_new_init_int(interp, enum_class_Integer, value); \\
+            VTABLE_set_attr_str(interp, pmc, \\
+                              Parrot_str_new_constant(interp, "$attrname"), attr_value); \\
+EOA
+        }
+        elsif ($attrtype eq "FLOATVAL") {
+            $decl .= <<"EOA";
+            PMC * const attr_value = Parrot_pmc_new(interp, enum_class_Float); \\
+            VTABLE_set_number_native(interp, attr_value, value); \\
+            VTABLE_set_attr_str(interp, pmc, \\
+                              Parrot_str_new_constant(interp, "$attrname"), attr_value); \\
+EOA
+        }
+        elsif ($attrtype =~ $isptrtostring) {
+            $decl .= <<"EOA";
+            PMC * const attr_value = Parrot_pmc_new(interp, enum_class_String); \\
+            VTABLE_set_string_native(interp, attr_value, value); \\
+            VTABLE_set_attr_str(interp, pmc, \\
+                              Parrot_str_new_constant(interp, "$attrname"), attr_value); \\
+EOA
+        }
+        elsif ($attrtype =~ $isptrtopmc) {
+            $decl .= <<"EOA";
+            VTABLE_set_attr_str(interp, pmc, \\
+                              Parrot_str_new_constant(interp, "$attrname"), value); \\
+EOA
+        }
+
+        else {
+            $decl .= <<"EOA";
+            Parrot_ex_throw_from_c_noargs(interp, EXCEPTION_INVALID_OPERATION, \\
+                "Attributes of type '$attrtype' cannot be " \\
+                "subclassed from a high-level PMC."); \\
+EOA
+        }
+
+        $decl .= <<"EOA";
+        } \\
+        else \\
+            ((Parrot_${pmcname}_attributes *)PMC_data(pmc))->$attrname = (value); \\
+    } while (0)
+
+EOA
+
+    }
+    #my $assertion = ($attrtype =~ $isptrtopmc and not $isfuncptr)
+    #    ? 'PARROT_ASSERT_INTERP((PMC *)(value), interp);'
+    #    : '';
+    #$decl .= <<"EOA";
+    #    } \\
+    #    else {\\
+    #        $assertion \\
+    #        ((Parrot_${pmcname}_attributes *)PMC_data(pmc))->$attrname = (value); \\
+    #    } \\
+    #} while (0)
+#EOA
+
+    $attribute->{inherit} = $inherit;
+
+    $pmc->{emitter}->emit($decl);
+
+    return 1;
+}
+
 
 1;
 
