@@ -1,4 +1,4 @@
-# Copyright (C) 2004-2011, Parrot Foundation.
+# Copyright (C) 2004-2012, Parrot Foundation.
 
 package Parrot::Pmc2c::Parser;
 
@@ -7,11 +7,9 @@ use warnings;
 use base qw( Exporter );
 our @EXPORT_OK = qw( parse_pmc extract_balanced );
 use Parrot::Pmc2c::PMC ();
-use Parrot::Pmc2c::Attribute ();
 use Parrot::Pmc2c::Method ();
 use Parrot::Pmc2c::Emitter ();
 use Parrot::Pmc2c::PCCMETHOD ();
-use Parrot::Pmc2c::MULTI ();
 use Parrot::Pmc2c::UtilFunctions qw(count_newlines filename slurp);
 use File::Basename qw(basename);
 
@@ -168,22 +166,21 @@ sub find_attrs {
 
     while ($pmcbody =~ s/($attr_re)//o) {
         my ($type, $name, $array_size, @modifiers, $comment);
-        $type = $2;
-        $name = $3;
+        $type = $2 || '';
+        $name = $3 || '';
         $array_size = $4 || '';
+        @modifiers = ();
         @modifiers = split /\s/, $5;
         $comment = $6;
 
         $lineno += count_newlines($1);
 
-        $pmc->add_attribute(Parrot::Pmc2c::Attribute->new(
-            {
+        $pmc->add_attribute( {
                 name       => $name,
                 type       => $type,
                 array_size => $array_size,
                 modifiers  => \@modifiers,
-            }
-        ));
+        } );
     }
 
     return ($lineno, $pmcbody);
@@ -247,6 +244,11 @@ sub find_methods {
         # trim trailing ws from last line
         $methodblock =~ s/\n[\t ]+$/\n/g;
 
+        # detect manual_wb via PARROT_GC_WRITE_BARRIER automatically
+        if ($methodblock =~ m|^\s*(/* no )?PARROT_GC_WRITE_BARRIER|m) {
+            $attrs->{manual_wb} = 1;
+        }
+
         $decorators ||= '';
         $decorators   =~ s/^\s*(.*?)\s*$/$1/s;
         $decorators   = [ split /\s+/ => $decorators ];
@@ -270,17 +272,20 @@ sub find_methods {
 
         # METHOD needs FixedIntegerArray header
         if ( $method->type eq Parrot::Pmc2c::Method::NON_VTABLE ) {
+            # rewrite_pccmethod() modifies $method in-place
             Parrot::Pmc2c::PCCMETHOD::rewrite_pccmethod( $method, $pmc );
             $pmc->set_flag('need_fia_header');
         }
         elsif ( $method->type eq Parrot::Pmc2c::Method::MULTI ) {
-            Parrot::Pmc2c::MULTI::rewrite_multi_sub( $method, $pmc );
+            # rewrite_multi_sub() modifies $method in-place
+            Parrot::Pmc2c::PCCMETHOD::rewrite_multi_sub( $method, $pmc );
         }
 
         if ( $method->type eq Parrot::Pmc2c::Method::NON_VTABLE
         ||   $method->type eq Parrot::Pmc2c::Method::MULTI ) {
             # Name-mangle NCI and multi methods to avoid conflict with vtables
-            Parrot::Pmc2c::PCCMETHOD::mangle_name( $method, $pmc );
+            # mangle_name() modifies $method in-place
+            Parrot::Pmc2c::PCCMETHOD::mangle_name( $method );
         }
 
         # PCCINVOKE needs FixedIntegerArray header
@@ -561,6 +566,7 @@ sub parse_method_attrs {
 
     my %result;
     ++$result{$1} while $flags =~ /:(\w+)/g;
+    $result{manual_wb}++ if $result{no_wb};
 
     return \%result;
 }
