@@ -425,11 +425,14 @@ sub convert_line_endings {
     return;
 }
 
+# Works only if not installed.
+# Using $PConfig{build_dir} would be much better
 sub path_to_parrot {
 
-    my $path = $INC{'Parrot/Config.pm'};
-    $path =~ s{ /lib/Parrot/Config.pm \z}{}xms;
-
+    my $path = __FILE__; # lib/Parrot/Test.pm
+    $path =~ s{\Qlib/Parrot/Test.pm\E$}{};
+    $path =~ s{/$}{};
+    $path = "." unless $path;
     return Cwd::realpath( $path );
 }
 
@@ -581,7 +584,7 @@ sub pbc_postprocess_output_like {
 }
 
 sub _pir_stdin_output_slurp {
-    my ($input_string, $code, $expected_ouptut) = @_;
+    my ($input_string, $code) = @_;
 
     my $stuff = sub {
         # Put the string on a file.
@@ -607,7 +610,7 @@ sub _pir_stdin_output_slurp {
             or die "Unable to pipe output to us: $!";
         <$in>;
     };
-    $result =~ s/(^==\d+==.*\n)//mg if defined $ENV{VALGRIND};
+    $result = valgrind_filter(undef, $result) if $ENV{VALGRIND};
     return $result;
 }
 
@@ -632,8 +635,6 @@ sub pir_stdin_output_is {
 Runs the PIR code while piping data into its standard input and passes the test
 if the output matches the expected result.
 
-=back
-
 =cut
 
 sub pir_stdin_output_like {
@@ -643,6 +644,30 @@ sub pir_stdin_output_like {
     Test::More::like($result, $expected_output, $description);
 }
 
+=item C<valgrind_filter($builder, $result)>
+
+Filters valgrind stderr output from the result string.
+
+=back
+
+=cut
+
+sub valgrind_filter {
+    my ($builder, $output) = @_;
+    if ($output =~ /^(?:==|--)\d+(?:==|--)\s+definitely lost: (\d+) bytes in (\d+) blocks\n/m) {
+        if ($1) {
+            my $msg = "LEAK SUMMARY: definitely lost: $1 bytes in $2 blocks";
+            $builder ? $builder->diag($msg) : Test::More::diag($msg);
+        }
+    }
+    $output =~ s/(^(?:==|--)\d+(?:==|--).*\n)//mg;
+    # --pid:0:syswrap- WARNING: Ignoring sigreturn( ..., UC_RESET_ALT_STACK );
+    # on longjmp, darwin only.
+    $output =~ s/(^--(?:\d)+:0:syswrap-.*\n)//mg
+      if $^O eq 'darwin' and !$ENV{TEST_VERBOSE};
+    $output;
+}
+
 # The following methods are private. They should not be used by modules
 # inheriting from Parrot::Test.
 
@@ -674,6 +699,9 @@ sub _run_test_file {
     # Switch to a different extension when we are generating code.
     my $out_f = per_test( '.out', $test_no );
 
+    # honor opt* filename to actually run code with -Ox
+    my $args = $ENV{TEST_PROG_ARGS} || '';
+
     # Name of the file with test code.
     # This depends on which kind of code we are testing.
     my $code_f;
@@ -682,9 +710,11 @@ sub _run_test_file {
     }
     elsif ( $func =~ m/^pasm_(exit_code|.*?output_)/ ) {
         $code_f = per_test( '.pasm', $test_no );
+        $args .= " -a";
     }
     elsif ( $func =~ m/^pbc_(exit_code|.*?output_)/ ) {
         $code_f = per_test( '.pbc', $test_no );
+        $code_f = $code if $code =~ /\.pbc$/ and -f $code;
     }
     else {
         die "Unknown test function: $func";
@@ -692,13 +722,11 @@ sub _run_test_file {
     $code_f = File::Spec->rel2abs($code_f);
     my $code_basef = basename($code_f);
 
-    # native tests are just run, others need to write code first
-    if ( $code_f !~ /\.pbc$/ ) {
+    # existing pbc test files are just run, need to write code first
+    if ( $code_f !~ /\.pbc$/ or ! -f $code_f ) {
         write_code_to_file( $code, $code_f );
     }
 
-    # honor opt* filename to actually run code with -Ox
-    my $args = $ENV{TEST_PROG_ARGS} || '';
     my $opt = $code_basef =~ m!opt(.)! ? "-O$1" : "";
     $args .= " $opt";
 
@@ -796,11 +824,7 @@ sub _generate_test_functions {
             my ( $code, $expected, $desc, %extra ) = @_;
             my $args                               = $ENV{TEST_PROG_ARGS} || '';
 
-            # Due to ongoing changes in PBC format, all tests in
-            # t/native_pbc/*.t are currently being SKIPped. This means we
-            # have no tests on which to model tests of the following block.
-            # Hence, test coverage will be lacking.
-            if ( $func =~ /^pbc_output_/ && $args =~ /-r / ) {
+            if ( $code =~ m|t/native_pbc/| && $func =~ /^pbc_output_/ && $args =~ /-r / ) {
                 # native tests with --run-pbc don't make sense
                 return $builder->skip("no native tests with -r");
             }
@@ -809,9 +833,8 @@ sub _generate_test_functions {
 
             my $meth        = $parrot_test_map{$func};
             my $real_output = slurp_file($out_f);
-            my $ori_output = $real_output;
-            $real_output =~ s/(^==\d+==.*\n)//mg if defined $ENV{VALGRIND};
-
+            my $ori_output  = $real_output;
+            $real_output = valgrind_filter($builder, $real_output) if $ENV{VALGRIND};
             _unlink_or_retain( $out_f );
 
             # set a todo-item for Test::Builder to find
@@ -824,12 +847,17 @@ sub _generate_test_functions {
 
             if ( $func =~ /_exit_code_is$/ ) {
                 $expected = int($expected);
+                if ($exit_code and $exit_code =~ /\[SIGNAL (\d+)\]/) {
+                    $exit_code = $1;
+                }
                 if ($exit_code == $expected) {
                     my $pass = $builder->$meth( $exit_code, $expected, $desc );
                     return $pass;
                 }
                 else {
-                    $builder->ok(0);
+                    $builder->ok(0, $desc);
+                    $builder->diag( "$cmd\nExited with error code: $exit_code, expected: $expected\n" );
+                    $builder->diag( "STDOUT:\n$real_output\n" ) if $ENV{TEST_VERBOSE};
                     return 0;
                 }
             }
@@ -840,11 +868,12 @@ sub _generate_test_functions {
             }
             elsif ($exit_code) {
                 $builder->ok( 0, $desc );
-                $builder->diag( "Exited with error code: $exit_code\n"
+                $builder->diag( "$cmd\nExited with error code: $exit_code\n"
                         . "Received:\n$real_output\nExpected:\n$expected\n"
                         . $ori_output ne $real_output ? "$ori_output\n" : "");
                 return 0;
             }
+            $cmd = join"\n",@$cmd if ref $cmd eq 'ARRAY';
             my $pass = $builder->$meth( $real_output, $expected, $desc );
             $builder->diag("'$cmd' failed with exit code $exit_code")
                 if not $pass and $exit_code;
@@ -980,6 +1009,7 @@ sub _generate_test_functions {
 
             # $test_no will be part of temporary files
             my $test_no = $builder->current_test() + 1;
+            my $verbose = $ENV{TEST_VERBOSE} ? $ENV{TEST_VERBOSE} : 0;
 
             convert_line_endings($expected);
 
@@ -1010,6 +1040,7 @@ sub _generate_test_functions {
                 );
                 $builder->diag("'$cmd' failed with exit code $exit_code")
                     if $exit_code;
+                $builder->diag( $cmd."\n" ) if $verbose > 1;
 
                 if ( !-e $obj_f ) {
                     $builder->diag( "Failed to build '$obj_f': " . slurp_file($build_f) );
@@ -1042,6 +1073,7 @@ sub _generate_test_functions {
                 );
                 $builder->diag("'$cmd' failed with exit code $exit_code")
                     if $exit_code;
+                $builder->diag( $cmd."\n" ) if $verbose > 1;
 
                 if ( !-e $exe_f ) {
                     $builder->diag( "Failed to build '$exe_f': " . slurp_file($build_f) );
@@ -1063,8 +1095,7 @@ sub _generate_test_functions {
                 );
                 my $output = slurp_file($out_f);
                 my $ori_output = $output;
-                $output =~ s/(^==\d+==.*\n)//mg if defined $ENV{VALGRIND};
-
+                $output = valgrind_filter($builder, $output) if $ENV{VALGRIND};
                 if ($exit_code) {
                     $pass = $builder->ok( 0, $desc );
                     $builder->diag( "Exited with error code: $exit_code\n"
@@ -1126,6 +1157,14 @@ sub _handle_blib_path {
     elsif ($^O eq 'MSWin32') {
         $ENV{PATH} = $blib_path . ';' . $ENV{PATH};
     }
+    elsif ($^O eq 'darwin') {
+        if (!$ENV{DYLD_LIBRARY_PATH}) {
+            $ENV{DYLD_LIBRARY_PATH} = $blib_path;
+        }
+        elsif ($ENV{DYLD_LIBRARY_PATH} !~ m|$blib_path|) {
+            $ENV{DYLD_LIBRARY_PATH} = "$blib_path:$ENV{DYLD_LIBRARY_PATH}";
+        }
+    }
     else {
         $ENV{LD_RUN_PATH} = $blib_path;
     }
